function [t,y] = TwoStepAdamsMoulton(f,t0,T,y0,N,M)

% function 2StepAdamsBashforth
%
% Zweck: integriere die gewoehnliche Diff.-Gleichung erster Ordnung
%
%          dy/dt = f(t,y(t))
%
%        mit einem 2-schritt Adams-Moulton Verfahren
%
% Parameters: f    ... rechte Seite f(t,y(t)) der gew. Diff.-Gl.
%             t0,T ... Start- und End-Zeit
%             y0   ... Anfagswert
%             N    ... Number of steps
%             M    ... Maximum number of iterations for the corrector
%
% Returns: t ... Zeiten
%          y ... approx. Loesung zu Zeiten t
%
% Notes: Keine.
%

% Schritt und Zeit
    t = zeros(N+1,1);
    h = T/N;
    for n=1:N+1
        t(n) = t0 + h*(n-1);
    end

% Initialisiere Loesungs-Vektor
    y = zeros(N+1,1);
    y(1) = y0;

% Erste schritt mit Heun Verfahren
    y(2) = heun_schritt(f, y(1), t(1), h);

% Loese: t(3) --> t(N+1)
    for n=3:N+1
        
        y_itr = zeros(M+1,1);
        
        % Predictor: Adams-Bashforth
        y_itr(1) = TwoStepAdamsBashforth(f, t(n-1), y(n-1), t(n-2), y(n-2), h);
        
        % Corrector: Adams-Moulton
        for k=1:M
            y_itr(k+1) = y(n-1) + h/12*( 5*f(t(n), y_itr(k)) + 8*f(t(n-1), y(n-1)) - f(t(n-2), y(n-2)));
        end
        
        y(n) = y_itr(M+1);
    end
    
end

% Heun methode
function yjp = heun_schritt(f,yj,tj,hj)

    k1 = f(tj,yj);
    k2 = f(tj + hj, yj + hj*k1);
    yjp = yj +  0.5*hj*(k1 + k2);
        
end

% Adams-Bashforth
function yjp = TwoStepAdamsBashforth(f,tj,yj,tjm,yjm,hj)

    yjp = yj + hj/2*( 3*f(tj, yj) - f(tjm, yjm) );

end

